@@ -24,6 +24,7 @@ using Do.Universe;
 
 namespace Do.Addins
 {
+	public delegate IPreferences GetPreferencesDelegate (string id);
 	public delegate void EnvironmentOpenDelegate (string item);
 	public delegate void PopupMainMenuAtPositionDelegate (int x, int y);
 	public delegate Gdk.Pixbuf PixbufFromIconNameDelegate (string icon_name, int size);
@@ -31,13 +32,16 @@ namespace Do.Addins
 	public delegate string FormatCommonSubstringsDelegate (string main, string highlight, string format);
 	public delegate void PresentWindowDelegate (Gtk.Window window);
 	public delegate void DoEventKeyDelegate (Gdk.EventKey key);
+	public delegate void NullEventHandler ();
+	public delegate void SearchStartedEventHandler (bool upstream_search);
+	public delegate void SearchFinishedEventHandler (bool selection_changed);
 	
 	/// <summary>
 	/// Useful functionality for plugins. See <see cref="Do.Util"/>.
 	/// </summary>
-	public class Util
+	public static class Util
 	{
-		
+		public static GetPreferencesDelegate GetPreferences;
 		public static FormatCommonSubstringsDelegate FormatCommonSubstrings;
 		
 		public static class Environment
@@ -45,7 +49,7 @@ namespace Do.Addins
 			public static EnvironmentOpenDelegate Open;
 		}
 		
-		public class Appearance
+		public static class Appearance
 		{
 			public static PresentWindowDelegate PresentWindow;
 			public static PixbufFromIconNameDelegate PixbufFromIconName;
