@@ -19,6 +19,7 @@
 
 using System;
 using System.Runtime.InteropServices;
+using System.Text;
 
 using Gtk;
 using Gdk;
@@ -28,7 +29,8 @@ using Do.Universe;
 
 namespace Do.UI
 {
-	public delegate void OnSelectionChanged (object sender, ResultsWindowSelectionEventArgs args);
+	public delegate void OnSelectionChanged (object sender, 
+	                                         ResultsWindowSelectionEventArgs args);
 
 	public class ResultsWindow : Gtk.Window
 	{
@@ -36,6 +38,8 @@ namespace Do.UI
 		private int DefaultWindowWidth = 352;
 		private int NumberResultsDisplayed = 6;
 		
+		private int offset;
+		
 		public string ResultInfoFormat
 		{
 			set { resultInfoFormat = value; }
@@ -56,14 +60,19 @@ namespace Do.UI
 		ScrolledWindow resultsScrolledWindow;
 		TreeView resultsTreeview;
 		IObject[] results;
-		int selectedIndex;
-		bool selectedIndexSet;
-		bool quietSelectionChange;
-		Label queryLabel;
+		int startResult, endResult;
 		Frame frame;
 		string query;
 		Gdk.Color backgroundColor;
 		VBox vbox;
+		Toolbar toolbar;
+		Label resultsLabel, queryLabel;
+		IUIContext context = null;
+		
+		int cursor;
+		int[] secondary = new int[0];
+		
+		bool pushedUpdate, clearing, update_needed = false;
 
 
 		public ResultsWindow (Gdk.Color backgroundColor, int NumberResults) 
@@ -73,10 +82,7 @@ namespace Do.UI
 			this.NumberResultsDisplayed = NumberResults;
 			
 			Build ();
-			results = null;
-			selectedIndex = 0;
-			selectedIndexSet = false;
-			Shown += OnShown;
+			results = new IObject[0];
 		}
 		
 		public ResultsWindow (Gdk.Color backgroundColor, int DefaultIconSize, 
@@ -89,28 +95,17 @@ namespace Do.UI
 			this.NumberResultsDisplayed = NumberResults;
 			
 			Build ();
-			results = null;
-			selectedIndex = 0;
-			selectedIndexSet = false;
-			Shown += OnShown;
-		}
-
-		protected virtual void OnShown (object sender, EventArgs args)
-		{
-			// setting Results calls Clear(), resets this.
-			int savedSelectedIndex;    
-
-			// Do this to load the icons.
-			savedSelectedIndex = selectedIndex;
-			quietSelectionChange = true;
-			Results = results;
-			selectedIndexSet = false;
-			SelectedIndex = savedSelectedIndex;
-			quietSelectionChange = false;
+			results = new IObject[0];
 		}
 
 		private void NotifySelectionChanged ()
 		{
+			TreeIter iter;
+			if (!resultsTreeview.Selection.GetSelected (out iter)) return;
+			try {
+				cursor = Convert.ToInt32 (resultsTreeview.Model.GetStringFromIter (iter));
+			} catch { return; }
+			
 			ResultsWindowSelectionEventArgs args;
 
 			args = new ResultsWindowSelectionEventArgs (SelectedIndex, SelectedObject);
@@ -124,17 +119,16 @@ namespace Do.UI
 			Alignment align;
 			TreeViewColumn column;
 			CellRenderer   cell;
+			HBox hbox;
 
 			KeepAbove = true;
 			AppPaintable = true;
 			AcceptFocus = false;
 			// This typehint gets the window to raise all the way to top.
 			TypeHint = WindowTypeHint.Splashscreen;
-
 			
 			SetColormap ();
 			
-			
 			frame = new Frame ();
 			frame.DrawFill = true;
 			frame.DrawFrame = true;
@@ -147,22 +141,28 @@ namespace Do.UI
 			vbox = new VBox (false, 0);
 			Add (frame);
 			frame.Add (vbox);
-			vbox.BorderWidth = 4;
+			vbox.BorderWidth = 3;
 			vbox.Show ();
-
-			align = new Alignment (0.0F, 0.0F, 0, 0);
-			align.SetPadding (1, 2, 1, 1);
-			queryLabel = new Label ();
-			queryLabel.UseMarkup = true;
-			queryLabel.SingleLineMode = true;
-			align.Add (queryLabel);
-			vbox.PackStart (align, false, false, 0);
-			// queryLabel.Show ();
-			// align.Show ();
 			
+			//---------The breadcrum bar---------
+			hbox = new HBox ();
+			toolbar = new Toolbar ();
+			align = new Alignment (0, .5f, 0, 0);
+			resultsLabel = new Label ();
+			queryLabel = new Label ();
+			align.Add (queryLabel);
+			hbox.PackStart (align, true, true, 4);
+			hbox.PackStart (resultsLabel, false, false, 0);
+			hbox.WidthRequest = DefaultWindowWidth - 10;
+			toolbar.Add (hbox);
+			toolbar.ShowAll ();
+			vbox.PackStart (toolbar, false, false, 0);
+			
+			
+			//---------Results Window
 			resultsScrolledWindow = new ScrolledWindow ();
-			resultsScrolledWindow.SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
-			resultsScrolledWindow.ShadowType = ShadowType.In;
+			resultsScrolledWindow.SetPolicy (PolicyType.Never, PolicyType.Never);
+			resultsScrolledWindow.ShadowType = ShadowType.None;
 			vbox.PackStart (resultsScrolledWindow, true, true, 0);
 			resultsScrolledWindow.Show ();
 
@@ -178,7 +178,7 @@ namespace Do.UI
 
 			resultsTreeview.Model = new ListStore (new Type[] {
 				typeof (IObject),				
-				typeof (string)
+				typeof (string), 
 			});
 
 			column = new TreeViewColumn ();			
@@ -194,21 +194,21 @@ namespace Do.UI
 			column.PackStart (cell, false);
 			column.SetCellDataFunc (cell, new TreeCellDataFunc (IconDataFunc));
 				
-			Console.WriteLine(height);
 			vbox.SetSizeRequest (DefaultWindowWidth, 
-				                 (height + 4) * NumberResultsDisplayed + 10);
+			                     (height + 2) * NumberResultsDisplayed + 
+			                     (int) (vbox.BorderWidth * 2) + 20);
 			
 			cell = new CellRendererText ();
 			(cell as CellRendererText).Ellipsize = Pango.EllipsizeMode.End;
 			column.PackStart (cell, true);
 			column.AddAttribute (cell, "markup", (int) Column.NameColumn);
-				
 			
-
 			resultsTreeview.AppendColumn (column);
 
 			resultsTreeview.Selection.Changed += OnResultRowSelected;
+			Shown += OnShown;
 		}
+		
 			
 		public void UpdateColors (Gdk.Color backgroundColor)
 		{
@@ -216,18 +216,50 @@ namespace Do.UI
 			frame.FillColor = backgroundColor;
 		}
 						
-		private void IconDataFunc (TreeViewColumn column, CellRenderer cell, TreeModel model, TreeIter iter)
+		private void IconDataFunc (TreeViewColumn column, CellRenderer cell, 
+		                           TreeModel model, TreeIter iter)
 		{			
 			CellRendererPixbuf renderer = cell as CellRendererPixbuf;
 			IObject o = (resultsTreeview.Model as ListStore).GetValue (iter, 0) as IObject;
-			renderer.Pixbuf = IconProvider.PixbufFromIconName (o.Icon, DefaultResultIconSize);
+			bool isSecondary = false;
+			foreach (int i in secondary)
+				if (model.GetStringFromIter (iter) == i.ToString ())
+					isSecondary = true;
+			
+			Gdk.Pixbuf final;
+			if (isSecondary) {
+				using (Gdk.Pixbuf source = IconProvider.PixbufFromIconName (o.Icon, DefaultResultIconSize))
+				using (Gdk.Pixbuf emblem = IconProvider.PixbufFromIconName ("gtk-add", DefaultResultIconSize)) {
+					final = new Pixbuf (Colorspace.Rgb, 
+					                    true, 
+					                    8,
+					                    DefaultResultIconSize,
+					                    DefaultResultIconSize);
+					
+					source.CopyArea (0, 0, source.Width, source.Height, final, 0, 0);
+					
+					emblem.Composite (final, 
+					                  0, 
+					                  0, 
+					                  DefaultResultIconSize, 
+					                  DefaultResultIconSize, 
+					                  0, 
+					                  0, 
+					                  1,
+					                  1, 
+					                  InterpType.Bilinear, 
+					                  220);
+				}
+			} else {
+				final = IconProvider.PixbufFromIconName (o.Icon, DefaultResultIconSize);
+			}
+			renderer.Pixbuf = final;
+			final.Dispose ();
 		}
 
 		private void OnResultRowSelected (object sender, EventArgs args)
 		{
-			if (!selectedIndexSet || quietSelectionChange) return;
-			if (resultsTreeview.Selection.GetSelectedRows().Length > 0) {
-				selectedIndex = resultsTreeview.Selection.GetSelectedRows()[0].Indices[0];
+			if (!clearing && !pushedUpdate) {
 				NotifySelectionChanged ();
 			}
 		}
@@ -235,132 +267,169 @@ namespace Do.UI
 		public virtual void Clear ()
 		{
 			(resultsTreeview.Model as ListStore).Clear ();
-			selectedIndex = 0;
-			selectedIndexSet = false;
-			results = null;
-			// Query = "";
+			cursor = 0;
+			resultsLabel.Markup = "--/--";
+			queryLabel.Markup = string.Empty;
+			update_needed = false;
 		}
 
-		public SearchContext Context
+		public IUIContext Context
 		{
 			set {
-				if (value == null) return;
-
-				if (Results.GetHashCode () != value.Results.GetHashCode ()) {
-					Results = value.Results;
-					Query = value.Query;
+				context = value;
+				if (!Visible) {
+					update_needed = true;
+					return;
 				}
-				SelectedIndex = value.Cursor;
+				
+				pushedUpdate = true;
+				if (value == null || value.Results.Length == 0) {
+					Results = new IObject [0];
+					return;
+				}
+				
+				if (results.GetHashCode () != value.Results.GetHashCode ()) {
+					results = value.Results;
+				}
+				
+				startResult = value.Cursor - 5;
+				
+				if (startResult < 0)
+					startResult = 0;
+				endResult = startResult + 8;
+				offset = startResult;
+				
+				if (endResult > results.Length)
+					endResult = results.Length;
+				
+				IObject[] resultsArray = new IObject[endResult - startResult];
+				Array.Copy (results, startResult, resultsArray, 0, resultsArray.Length); 
+				Results = resultsArray;
+				Query = value.Query;
+				
+				cursor = value.Cursor - offset;
+				
+				int[] secArray = new int[value.SecondaryCursors.Length];
+				for (int i=0; i<secArray.Length; i++) {
+					secArray[i] = value.SecondaryCursors[i] - offset;
+				}
+				
+				secondary = secArray;
+				
+				
+				UpdateCursors ();
+				UpdateQueryLabel (value);
+				resultsLabel.Markup = string.Format ("{1}/{0}", 
+				                                     value.Results.Length, 
+				                                     value.Cursor + 1);
+				Gtk.Application.Invoke (delegate {
+					pushedUpdate = false;
+				});
 			}
 		}
 
 		public int SelectedIndex
 		{
-			get { return selectedIndex; }
-			set {
-				TreeModel model;
-				TreeIter iter;
-				TreePath path;
-				int new_selection;
-
-				if (selectedIndexSet &&
-				    value == selectedIndex) return;
-
-				selectedIndexSet = true;
-
-				// Don't bother updating widgets if we're not visible -
-				// instead, do these things on Show.
-				if (!Visible) {
-					selectedIndex = value;
-					return;
-				}
-
-				if (results.Length == 0)
-					return;
-				else if (value >= results.Length)
-					new_selection = results.Length - 1;
-				else if (value < 0)
-					new_selection = 0;
-				else
-					new_selection = value;
-
-				resultsTreeview.Selection.GetSelected (out model, out iter);
-				path = model.GetPath (iter);
-				// TODO: Just jump to new index instead of iterating like this.
-				while (new_selection > selectedIndex) {
-					selectedIndex++;
-					path.Next ();
-				}
-				while (new_selection < selectedIndex) {
-					selectedIndex--;
-					path.Prev ();
-				}
-				resultsTreeview.Selection.SelectPath (path);
-				resultsTreeview.ScrollToCell (path, null, false, 0.0F, 0.0F);
+			get { return cursor + offset; }
+			set { 
+				cursor = value - offset;
+				
+				UpdateCursors ();
 			}
 		}
+			
+		private void UpdateQueryLabel (IUIContext context)
+		{
+			string query = context.Query;
+			StringBuilder builder = new StringBuilder ();
+			
+			int count = 0;
+			while (context.ParentContext != null && count < 2) {
+				builder.Insert (0, context.ParentContext.Selection.Name + " > ");
+				context = context.ParentContext;
+				count++;
+			}
+			queryLabel.Markup = string.Format ("{0}<b>{1}</b>", 
+			                                   GLib.Markup.EscapeText (builder.ToString ()),
+			                                   GLib.Markup.EscapeText (query));
+		}
+		
+		private void UpdateCursors () 
+		{
+			resultsTreeview.Selection.UnselectAll ();
 
+			Gtk.TreePath path;
+			
+			path = new TreePath (cursor.ToString ());
+			
+			//makes this just a tiny bit smoother overall
+			Gtk.Application.Invoke (delegate {
+				resultsTreeview.ScrollToCell (path, null, true, 0.5F, 0.0F);
+				resultsTreeview.Selection.SelectPath (path);
+			});
+		}
+		
 		public IObject SelectedObject
 		{
 			get {
 				try {
-					return results[selectedIndex];
+					return results[SelectedIndex];
 				} catch {
 					return null;
 				}
 			}
 		}
-
+		
 		public IObject[] Results
 		{
-			//Needed for hashing
-			get { return results ?? results = new IObject[0]; }
-			set {				
+			set {
+				//some memory hacks.
+				foreach (CellRenderer rend in resultsTreeview.Columns[0].CellRenderers) {
+					if (rend is CellRendererPixbuf && (rend as CellRendererPixbuf).Pixbuf != null) {
+						(rend as CellRendererPixbuf).Pixbuf.Dispose ();
+					}
+					rend.Dispose ();
+				}
+				
 				ListStore store;
-				TreeIter iter, first_iter;
-				bool seen_first;				
 				string info;
 
+				clearing = true;
 				Clear ();
-				results = value ?? new IObject[0];
 				store = resultsTreeview.Model as ListStore;
-				first_iter = default (TreeIter);
-				seen_first = false;
 
-				// Don't bother updating widgets if we're not visible -
-				// instead, do these things on Show.
-				if (!Visible) return;
-
-				foreach (IObject result in results) {					
+				foreach (IObject result in value) {					
 					
-					info = string.Format (ResultInfoFormat, result.Name, result.Description);
-					info = Util.Appearance.MarkupSafeString (info);
-					iter = store.AppendValues (new object[] {
+					info = string.Format (ResultInfoFormat, 
+					                      GLib.Markup.EscapeText (result.Name), 
+					                      GLib.Markup.EscapeText (result.Description));
+					store.AppendValues (new object[] {
 						result,
-						info
+						info,
 					});
 							
-					if (!seen_first) {
-						first_iter = iter;
-						seen_first = true;
-					}
-				}
-				if (seen_first) {
-					resultsTreeview.ScrollToCell (resultsTreeview.Model.GetPath (first_iter),
-					                              null, false, 0.0F, 0.0F);
-					resultsTreeview.Selection.SelectIter (first_iter);
 				}
+				clearing = false;
 			}
 		}
 
+
 		public string Query
 		{
 			set {
 				query = value;
-				queryLabel.Markup = string.Format (QueryLabelFormat, value ?? "");
+				queryLabel.Markup = string.Format (QueryLabelFormat, 
+				                                   GLib.Markup.EscapeText (value ?? string.Empty));
 			}
 			get { return query; }
 		}
+		
+		protected void OnShown (object o, EventArgs args)
+		{
+			if (update_needed)
+				Context = context;
+			update_needed = false;
+		}
 
 		// Draw a border around the window.
 		protected override bool OnExposeEvent (EventExpose evnt)
@@ -387,6 +456,7 @@ namespace Do.UI
 				Console.Error.WriteLine ("No alpha support.");
 			}
 			Colormap = colormap;
+			colormap.Dispose ();
 		}
 	}
 }
