@@ -26,15 +26,12 @@ using Do.Addins;
 using Gdk;
 using Gtk;
 
-namespace Do.UI
-{
+namespace Do.UI {
 	
-	
-	public class ClassicWindow : Gtk.Window, IDoWindow
-	{
+	public class ClassicWindow : Gtk.Window, IDoWindow {
 		
 		//-------------------Class Members------------------
-		GlossyRoundedFrame frame;
+		Frame frame;
 		SymbolDisplayLabel label;
 		ResultsWindow resultsWindow;
 		PositionWindow positionWindow;
@@ -51,17 +48,11 @@ namespace Do.UI
 		const double WindowTransparency = 0.91;
 		
 		Pane currentPane;
-		bool summonable;
 		
 		//-------------------Events-----------------------
 		public new event DoEventKeyDelegate KeyPressEvent;
 			
 		//-------------------Properties-------------------
-		public bool IsSummonable {
-			get {
-				return summonable;
-			}
-		}
 
 		public Pane CurrentPane {
 			get {
@@ -179,25 +170,16 @@ namespace Do.UI
 
 			ScreenChanged += OnScreenChanged;
 			ConfigureEvent += OnConfigureEvent;
-			
-			summonable = true;
-
+			SizeAllocated += delegate { Reposition (); };
+			iconbox[0].LinesChanged += OnLineChangedEvent;
+			iconbox[1].LinesChanged += OnLineChangedEvent;
+			iconbox[2].LinesChanged += OnLineChangedEvent;
 			Reposition ();
 		}
 		
 		protected override bool OnKeyPressEvent (EventKey evnt)
 		{
-			switch (evnt.Key) {
-				case Gdk.Key.Page_Up:
-					resultsWindow.SelectedIndex -= NumberResultsDisplayed;
-					break;
-				case Gdk.Key.Page_Down:
-					resultsWindow.SelectedIndex += NumberResultsDisplayed;
-					break;
-				default:
-					KeyPressEvent (evnt);
-					break;
-			}
+			KeyPressEvent (evnt);
 
 			return base.OnKeyPressEvent (evnt);
 		}
@@ -213,6 +195,7 @@ namespace Do.UI
 			}
 			
 			Colormap = colormap;
+			colormap.Dispose ();
 		}
 		
 		private Gdk.Color BackgroundColor
@@ -221,7 +204,9 @@ namespace Do.UI
 				byte r, g, b;
 				Gdk.Color bgColor;
 
-				bgColor = Gtk.Rc.GetStyle (this).Backgrounds[(int) StateType.Selected];
+				using (Gtk.Style style = Gtk.Rc.GetStyle (this)) {
+					bgColor = style.Backgrounds[(int) StateType.Selected];
+				}
 				r = (byte) ((bgColor.Red) >> 8);
 				g = (byte) ((bgColor.Green) >> 8);
 				b = (byte) ((bgColor.Blue) >> 8);
@@ -266,13 +251,23 @@ namespace Do.UI
 		
 		public virtual void Reposition ()
 		{
-			Gdk.Rectangle offset;
-			int iconboxWidth;
-
-			offset = new Rectangle (IconBoxRadius, 0, 0 ,0);
-			iconboxWidth = IconBoxIconSize + 60;
-			
-			PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			Gtk.Application.Invoke (delegate {
+				Gdk.Rectangle offset;
+				int iconboxWidth;
+				
+				offset = new Rectangle (IconBoxRadius, 0, 0 ,0);
+				iconboxWidth = IconBoxIconSize + 60;
+				
+				PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			});
+		}
+		
+		protected void OnLineChangedEvent (object o, EventArgs a)
+		{
+			if ((int) o <= 2) return;
+			this.QueueDraw ();
+			this.Resize (1, 1);
+			Reposition ();
 		}
 		
 		protected override bool OnButtonPressEvent (EventButton evnt)
@@ -313,9 +308,11 @@ namespace Do.UI
 		
 		public void Summon ()
 		{
-			if (PositionWindow.GetMonitor()) {
-				Reposition ();
-			}
+			frame.Radius = Screen.IsComposited ? IconBoxRadius : 0;
+
+			PositionWindow.GetMonitor ();
+			Resize (1, 1);
+			Reposition ();
 			Show ();
 			Util.Appearance.PresentWindow (this);
 		}
@@ -333,6 +330,10 @@ namespace Do.UI
 			iconbox[1].Clear ();
 			iconbox[2].Clear ();
 			
+			QueueDraw ();
+			Resize (1, 1);
+			Reposition ();
+			
 			iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
 			label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
 			                       Catalog.GetString ("Type to start searching."));
@@ -342,24 +343,12 @@ namespace Do.UI
 		{
 			iconbox[2].Show ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 
 		public void Shrink ()
 		{
 			iconbox[2].Hide ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 		
 		public void GrowResults ()
@@ -374,26 +363,49 @@ namespace Do.UI
 				resultsWindow.Hide ();
 		}
 		
-		public void SetPaneContext (Pane pane, SearchContext context)
+		public void SetPaneContext (Pane pane, IUIContext context)
 		{
-			if (context.Results.Length == 0) {
-				NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
-				for (int i = (int) pane; i < 3; i++) {
-					iconbox[i].Clear ();
-					iconbox[i].DisplayObject = noRes;
-					if (i == (int) CurrentPane) {
-						label.SetDisplayLabel (noRes.Name, noRes.Description);
-						resultsWindow.Context = context;
+			if (context.Results.Length == 0 && !context.LargeTextDisplay) {
+				if (pane == Pane.First && context.ParentContext == null) {
+					iconbox[0].TextOverlay = context.LargeTextDisplay;
+					iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
+					label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
+					                       Catalog.GetString ("Type to start searching."));
+				} else {
+					NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
+					for (int i = (int) pane; i < 3; i++) {
+						iconbox[i].Clear ();
+						iconbox[i].DisplayObject = noRes;
+						if (i == (int) CurrentPane) {
+							label.SetDisplayLabel (noRes.Name, noRes.Description);
+							resultsWindow.Clear ();
+						}
 					}
 				}
 				return;
 			}
-			iconbox[(int) pane].DisplayObject = context.Selection;
-			iconbox[(int) pane].Highlight = context.Query;
+			
+			if (context.Query.Length == 0 && context.LargeTextDisplay) {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = new TextItem ("Enter Text");
+				
+				if (context.Results.Length == 0) return;
+			} else {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = context.Selection;
+				
+				if (!context.LargeTextDisplay)
+					iconbox[(int) pane].Highlight = context.Query;
+			}
+			
+			if (context.Selection == null) return;
 			
 			if (pane == CurrentPane) {
 				resultsWindow.Context = context;
-				label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				if (!context.LargeTextDisplay)
+					label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				else
+					label.SetDisplayLabel ("", "Raw Text Mode");
 			}
 		}
 
