@@ -51,18 +51,11 @@ namespace Do.UI
 		const int MainRadius = 13;
 		
 		Pane currentPane;
-		bool summonable;
 		
 		//-------------------Events-----------------------
 		public new event DoEventKeyDelegate KeyPressEvent;
 			
 		//-------------------Properties-------------------
-		public bool IsSummonable {
-			get {
-				return summonable;
-			}
-		}
-		
 		/// <value>
 		/// IDoWindow public pane member
 		/// </value>
@@ -113,7 +106,6 @@ namespace Do.UI
 		protected void Build ()
 		{
 			VBox      vbox;
-//			Alignment align;
 
 			AppPaintable = true;
 			KeepAbove = true;
@@ -148,13 +140,7 @@ namespace Do.UI
 			vbox.BorderWidth = (uint) (IconBoxPadding + frameoffset);
 			vbox.Show ();
 			
-//			align = new Alignment (0.5F, 0.5F, 1, 1);
-//			align.SetPadding (0, 0, 0, 0);
 			label = new SymbolDisplayLabel ();
-//			align.Add (label);
-//			vbox.PackStart (align, false, false, 0);
-//			label.Show ();
-//			align.Show ();
 
 			resultsHBox = new HBox (false, (int) IconBoxPadding * 2);
 			resultsHBox.BorderWidth = IconBoxPadding;
@@ -166,27 +152,26 @@ namespace Do.UI
 			iconbox[0] = new GlassIconBox (IconBoxIconSize);
 			iconbox[0].IsFocused = true;
 			iconbox[0].Radius = IconBoxRadius;
-			resultsHBox.PackStart (iconbox[0], false, false, 0);
+			resultsHBox.PackStart (iconbox[0], true, true, 0);
 			iconbox[0].Show ();
 
 			iconbox[1] = new GlassIconBox (IconBoxIconSize);
 			iconbox[1].IsFocused = false;
 			iconbox[1].Radius = IconBoxRadius;
-			resultsHBox.PackStart (iconbox[1], false, false, 0);
+			resultsHBox.PackStart (iconbox[1], true, true, 0);
 			iconbox[1].Show ();
 
 			iconbox[2] = new GlassIconBox (IconBoxIconSize);
 			iconbox[2].IsFocused = false;
 			iconbox[2].Radius = IconBoxRadius;
-			resultsHBox.PackStart (iconbox[2], false, false, 0);
-
+			resultsHBox.PackStart (iconbox[2], true, true, 0);
 			
-
 			ScreenChanged += OnScreenChanged;
 			ConfigureEvent += OnConfigureEvent;
-			
-			summonable = true;
-
+			SizeAllocated += delegate { Reposition (); };
+			iconbox[0].LinesChanged += OnLineChangedEvent;
+			iconbox[1].LinesChanged += OnLineChangedEvent;
+			iconbox[2].LinesChanged += OnLineChangedEvent;
 			Reposition ();
 		}
 		
@@ -323,12 +308,12 @@ namespace Do.UI
 		protected override bool OnKeyPressEvent (EventKey evnt)
 		{
 			switch (evnt.Key) {
-				case Gdk.Key.Page_Up:
+				/*case Gdk.Key.Page_Up:
 					resultsWindow.SelectedIndex -= NumberResultsDisplayed;
 					break;
 				case Gdk.Key.Page_Down:
 					resultsWindow.SelectedIndex += NumberResultsDisplayed;
-					break;
+					break;*/
 				default:
 					KeyPressEvent (evnt);
 					break;
@@ -342,13 +327,23 @@ namespace Do.UI
 		/// </summary>
 		public void Reposition ()
 		{
-			Gdk.Rectangle offset;
-			int iconboxWidth;
-
-			offset = new Rectangle (frameoffset + ((int) IconBoxPadding * 2), 0, 0 ,0);
-			iconboxWidth = (iconbox[0].Width + ((int) IconBoxPadding * 4));
-			
-			PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			Gtk.Application.Invoke (delegate {
+				Gdk.Rectangle offset;
+				int iconboxWidth;
+				
+				offset = new Rectangle (IconBoxRadius, 0, 0 ,0);
+				iconboxWidth = IconBoxIconSize + 150;
+				
+				PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			});
+		}
+		
+		protected void OnLineChangedEvent (object o, EventArgs a)
+		{
+			if ((int) o <= 2) return;
+			this.QueueDraw ();
+			this.Resize (1, 1);
+			Reposition ();
 		}
 		
 		  ///////////////////////
@@ -358,9 +353,9 @@ namespace Do.UI
 		public void Summon ()
 		{
 			//needed to know where our monitor sits...
-			if (PositionWindow.GetMonitor ()) {
-				Reposition ();
-			}
+			PositionWindow.GetMonitor ();
+			Resize (1, 1);
+			Reposition ();
 			Show ();
 			Util.Appearance.PresentWindow (this);
 		}
@@ -384,6 +379,10 @@ namespace Do.UI
 			iconbox[1].Clear ();
 			iconbox[2].Clear ();
 			
+			QueueDraw ();
+			Resize (1, 1);
+			Reposition ();
+			
 			iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
 			label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
 			                       Catalog.GetString ("Type to start searching."));
@@ -396,12 +395,6 @@ namespace Do.UI
 		{
 			iconbox[2].Show ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 
 		/// <summary>
@@ -411,12 +404,6 @@ namespace Do.UI
 		{
 			iconbox[2].Hide ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 		
 		/// <summary>
@@ -448,36 +435,52 @@ namespace Do.UI
 		/// <param name="context">
 		/// A <see cref="SearchContext"/>
 		/// </param>
-		public void SetPaneContext (Pane pane, SearchContext context)
+		public void SetPaneContext (Pane pane, IUIContext context)
 		{
-			if (context.Results.Length == 0) {
-				NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
-				for (int i = (int) pane; i < 3; i++) {
-					iconbox[i].Clear ();
-					iconbox[i].DisplayObject = noRes;
-					if (i == (int) CurrentPane) {
-						label.SetDisplayLabel (noRes.Name, noRes.Description);
-						resultsWindow.Context = context;
+			if (context.Results.Length == 0 && !context.LargeTextDisplay) {
+				if (pane == Pane.First && context.ParentContext == null) {
+					iconbox[0].TextOverlay = context.LargeTextDisplay;
+					iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
+					label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
+					                       Catalog.GetString ("Type to start searching."));
+				} else {
+					NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
+					for (int i = (int) pane; i < 3; i++) {
+						iconbox[i].Clear ();
+						iconbox[i].DisplayObject = noRes;
+						if (i == (int) CurrentPane) {
+							label.SetDisplayLabel (noRes.Name, noRes.Description);
+							resultsWindow.Clear ();
+						}
 					}
 				}
 				return;
 			}
-			iconbox[(int) pane].DisplayObject = context.Selection;
-			iconbox[(int) pane].Highlight = context.Query;
+			
+			if (context.Query.Length == 0 && context.LargeTextDisplay) {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = new TextItem ("Enter Text");
+				
+				if (context.Results.Length == 0) return;
+			} else {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = context.Selection;
+				
+				if (!context.LargeTextDisplay)
+					iconbox[(int) pane].Highlight = context.Query;
+			}
+			
+			if (context.Selection == null) return;
 			
 			if (pane == CurrentPane) {
 				resultsWindow.Context = context;
-				label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				if (!context.LargeTextDisplay)
+					label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				else
+					label.SetDisplayLabel ("", "Raw Text Mode");
 			}
 		}
 
-		/// <summary>
-		/// Controller will occasionally ask for a specific pane to be cleared.  We must also
-		/// clear the results window if the pane that is cleared is the current pane.
-		/// </summary>
-		/// <param name="pane">
-		/// A <see cref="Pane"/>
-		/// </param>
 		public void ClearPane (Pane pane)
 		{
 			iconbox[(int) pane].Clear ();
