@@ -52,18 +52,11 @@ namespace Do.UI
 		const double WindowTransparency = 0.95;
 		
 		Pane currentPane;
-		bool summonable;
 		
 		//-------------------Events-----------------------
 		public new event DoEventKeyDelegate KeyPressEvent;
 			
 		//-------------------Properties-------------------
-		public bool IsSummonable {
-			get {
-				return summonable;
-			}
-		}
-
 		public Pane CurrentPane {
 			get {
 				return currentPane;
@@ -167,6 +160,7 @@ namespace Do.UI
 			iconbox[2].IsFocused = false;
 			iconbox[2].Radius = IconBoxRadius;
 			resultsHBox.PackStart (iconbox[2], false, false, 0);
+			
 			// iconbox[2].Show ();
 
 			align = new Alignment (0.5F, 0.5F, 1, 1);
@@ -179,21 +173,22 @@ namespace Do.UI
 
 			ScreenChanged += OnScreenChanged;
 			ConfigureEvent += OnConfigureEvent;
-			
-			summonable = true;
-
+			SizeAllocated += delegate { Reposition (); };
+			iconbox[0].LinesChanged += OnLineChangedEvent;
+			iconbox[1].LinesChanged += OnLineChangedEvent;
+			iconbox[2].LinesChanged += OnLineChangedEvent;
 			Reposition ();
 		}
 		
 		protected override bool OnKeyPressEvent (EventKey evnt)
 		{
 			switch (evnt.Key) {
-				case Gdk.Key.Page_Up:
+				/*case Gdk.Key.Page_Up:
 					resultsWindow.SelectedIndex -= NumberResultsDisplayed;
 					break;
 				case Gdk.Key.Page_Down:
 					resultsWindow.SelectedIndex += NumberResultsDisplayed;
-					break;
+					break;*/
 				default:
 					KeyPressEvent (evnt);
 					break;
@@ -227,13 +222,23 @@ namespace Do.UI
 		
 		public void Reposition ()
 		{		
-			Gdk.Rectangle offset;
-			int iconboxWidth;
-
-			offset = new Rectangle (MainRadius, 0, 0 ,0);
-			iconboxWidth = (iconbox[0].Width + ((int) IconBoxPadding * 2));
-			
-			PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			Gtk.Application.Invoke (delegate {
+				Gdk.Rectangle offset;
+				int iconboxWidth;
+				
+				offset = new Rectangle (IconBoxRadius, 0, 0 ,0);
+				iconboxWidth = IconBoxIconSize + 115;
+				
+				PositionWindow.UpdatePosition (iconboxWidth, currentPane, offset);
+			});
+		}
+		
+		protected void OnLineChangedEvent (object o, EventArgs a)
+		{
+			if ((int) o <= 2) return;
+			this.QueueDraw ();
+			this.Resize (1, 1);
+			Reposition ();
 		}
 		
 		protected override bool OnButtonPressEvent (EventButton evnt)
@@ -274,9 +279,9 @@ namespace Do.UI
 		
 		public void Summon ()
 		{
-			if (PositionWindow.GetMonitor ()) {
-				Reposition ();
-			}
+			PositionWindow.GetMonitor ();
+			Resize (1, 1);
+			Reposition ();
 			Show ();
 			Util.Appearance.PresentWindow (this);
 		}
@@ -294,6 +299,10 @@ namespace Do.UI
 			iconbox[1].Clear ();
 			iconbox[2].Clear ();
 			
+			QueueDraw ();
+			Resize (1, 1);
+			Reposition ();
+			
 			iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
 			label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
 			                       Catalog.GetString ("Type to start searching."));
@@ -303,24 +312,12 @@ namespace Do.UI
 		{
 			iconbox[2].Show ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 
 		public void Shrink ()
 		{
 			iconbox[2].Hide ();
 			Resize (1, 1);
-			GLib.Timeout.Add (70, delegate {
-				Gdk.Threads.Enter ();
-				Reposition ();
-				Gdk.Threads.Leave ();
-				return false;
-			});
 		}
 		
 		public void GrowResults ()
@@ -335,26 +332,49 @@ namespace Do.UI
 				resultsWindow.Hide ();
 		}
 		
-		public void SetPaneContext (Pane pane, SearchContext context)
+		public void SetPaneContext (Pane pane, IUIContext context)
 		{
-			if (context.Results.Length == 0) {
-				NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
-				for (int i = (int) pane; i < 3; i++) {
-					iconbox[i].Clear ();
-					iconbox[i].DisplayObject = noRes;
-					if (i == (int) CurrentPane) {
-						label.SetDisplayLabel (noRes.Name, noRes.Description);
-						resultsWindow.Context = context;
+			if (context.Results.Length == 0 && !context.LargeTextDisplay) {
+				if (pane == Pane.First && context.ParentContext == null) {
+					iconbox[0].TextOverlay = context.LargeTextDisplay;
+					iconbox[0].DisplayObject = new Do.Addins.DefaultIconBoxObject ();
+					label.SetDisplayLabel (Catalog.GetString ("Type to begin searching"), 
+					                       Catalog.GetString ("Type to start searching."));
+				} else {
+					NoResultsFoundObject noRes = new NoResultsFoundObject (context.Query);
+					for (int i = (int) pane; i < 3; i++) {
+						iconbox[i].Clear ();
+						iconbox[i].DisplayObject = noRes;
+						if (i == (int) CurrentPane) {
+							label.SetDisplayLabel (noRes.Name, noRes.Description);
+							resultsWindow.Clear ();
+						}
 					}
 				}
 				return;
 			}
-			iconbox[(int) pane].DisplayObject = context.Selection;
-			iconbox[(int) pane].Highlight = context.Query;
+			
+			if (context.Query.Length == 0 && context.LargeTextDisplay) {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = new TextItem ("Enter Text");
+				
+				if (context.Results.Length == 0) return;
+			} else {
+				iconbox[(int) pane].TextOverlay = context.LargeTextDisplay;
+				iconbox[(int) pane].DisplayObject = context.Selection;
+				
+				if (!context.LargeTextDisplay)
+					iconbox[(int) pane].Highlight = context.Query;
+			}
+			
+			if (context.Selection == null) return;
 			
 			if (pane == CurrentPane) {
 				resultsWindow.Context = context;
-				label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				if (!context.LargeTextDisplay)
+					label.SetDisplayLabel (context.Selection.Name, context.Selection.Description);
+				else
+					label.SetDisplayLabel ("", "Raw Text Mode");
 			}
 		}
 
