@@ -21,12 +21,15 @@
 using System;
 using System.IO;
 using System.Collections.Generic;
+using Mono.Unix;
 
 namespace Do.Universe {
 
 	public class ApplicationItemSource : IItemSource {
 
-		private List<IItem> apps;
+		private Dictionary<string,IItem> apps;
+
+		bool show_hidden = false;
 		
 		/// <summary>
 		/// Locations to search for .desktop files.
@@ -39,6 +42,7 @@ namespace Do.Universe {
 					"/usr/share/gdm/applications",
 					"/usr/local/share/applications",
 					"~/.local/share/applications",
+					"~/.local/share/applications/wine",
 					Desktop,
 				};
 			}
@@ -52,8 +56,7 @@ namespace Do.Universe {
 
 		public ApplicationItemSource ()
 		{
-			apps = new List<IItem> ();
-			UpdateItems ();
+			apps = new Dictionary<string,IItem> ();
 		}
 
 		public Type [] SupportedItemTypes {
@@ -65,11 +68,11 @@ namespace Do.Universe {
 		}
 
 		public string Name {
-			get { return "Applications"; }
+			get { return Catalog.GetString ("Applications"); }
 		}
 
 		public string Description {
-			get { return "Finds applications in many locations."; }
+			get { return Catalog.GetString ("Finds applications in many locations."); }
 		}
 
 		public string Icon {
@@ -89,35 +92,42 @@ namespace Do.Universe {
 		/// </param>
 		private void LoadDesktopFiles (string dir)
 		{
-			ApplicationItem app;
-
 			if (!Directory.Exists (dir)) return;
-			foreach (string filename in Directory.GetFiles (dir)) {
-				if (!filename.EndsWith (".desktop")) continue;
+			foreach (string file in Directory.GetFiles (dir, "*.desktop")) {
+                ApplicationItem app;
+
+                if (apps.ContainsKey (file)) continue;
 				try {
-					app = new ApplicationItem (filename);
+					app = new ApplicationItem (file);
 				} catch {
 					continue;
 				}
-				apps.Add (app);
+				if (string.IsNullOrEmpty (app.Exec) || string.IsNullOrEmpty (app.Name))
+					continue;
+				
+				if (!app.Hidden || show_hidden)
+					apps [file] = app;
 			}
 		}
 
 		public void UpdateItems ()
 		{
-			apps.Clear ();
+			// Updating is turned off because it uses a ridiculous amount of memory.
+			if (apps.Count > 0) return;
+			
 			foreach (string dir in DesktopFilesDirectories) {
 				LoadDesktopFiles (dir.Replace ("~", Paths.UserHome));
 			}
 		}
 
 		public ICollection<IItem> Items {
-			get { return apps; }
+			get { return apps.Values; }
 		}
 
 		public ICollection<IItem> ChildrenOfItem (IItem item)
 		{
 			return null;
 		}
+
 	}
 }
