@@ -32,15 +32,15 @@ namespace Do {
 
 		static Paths ()
 		{
-			if (File.Exists (Temp)) {
+			if (Directory.Exists (Temp)) {
 				try {
-					File.Delete (Temp);
+					Directory.Delete (Temp, true);
 				} catch (Exception e) {
-					Console.Error.WriteLine ("Could not delete temporary directory {0}: {1}",
+					Console.Error.WriteLine (
+						"Could not delete temporary directory {0}: {1}",
 						Temp, e.Message);
 				}
 			}
-
 			CreateDirs (ApplicationData, UserData, UserPlugins, Temp);
 		}
 
@@ -50,7 +50,8 @@ namespace Do {
 				try {
 					Directory.CreateDirectory (first);
 				} catch (Exception e) {
-					Console.Error.WriteLine ("Failed to create directory {0}: {1}",
+					Console.Error.WriteLine (
+						"Failed to create directory {0}: {1}",
 						first, e.Message);
 				}
 			}
@@ -60,7 +61,8 @@ namespace Do {
 					try {
 						Directory.CreateDirectory (dir);
 					} catch (Exception e) {
-						Console.Error.WriteLine ("Failed to create directory {0}: {1}",
+						Console.Error.WriteLine (
+							"Failed to create directory {0}: {1}",
 							dir, e.Message);
 					}
 				}
@@ -136,7 +138,6 @@ namespace Do {
 
 		public static string UserHome {
 			get {
-
 				string home = Environment.GetEnvironmentVariable ("HOME");
 				if (string.IsNullOrEmpty (home))
 					home = Environment.GetFolderPath (Environment.SpecialFolder.Personal);
@@ -147,7 +148,11 @@ namespace Do {
 		public static string Temp {
 			get { return Combine (Paths.ApplicationData, "tmp"); }
 		}
-
+		
+		public static string Log {
+			get { return Combine (Paths.ApplicationData, "Log"); }
+		}
+		
 		public static string GetTemporaryFilePath ()
 		{
 			int fileId;
@@ -172,7 +177,7 @@ namespace Do {
 		}
 
 		public static string UserPlugins {
-			get { return Combine (UserData, "plugins"); }
+			get { return Combine (UserData, "plugins-" + Version); }
 		}
 
 		public static string[] SystemData {
@@ -201,6 +206,16 @@ namespace Do {
 				return dirs;
 			}
 		}
+
+		private static string Version {
+			get {
+				System.Reflection.AssemblyName name;
+
+				name = typeof (Paths).Assembly.GetName ();
+				return string.Format ("{0}.{1}.{2}",
+					name.Version.Major, name.Version.Minor, name.Version.Build);
+			}
+		}
 	}
 }
 
