@@ -20,21 +20,14 @@ using System.Runtime.CompilerServices;
 [assembly: AssemblyTrademark("")]
 [assembly: AssemblyCulture("")]
 
-
-// This is text that goes after " [options]" in help output.
-[assembly: Mono.UsageComplement ("")]
-
-// Attributes visible in " -V"
-[assembly: Mono.About("Do things quickly.")]
-[assembly: Mono.Author ("David Siegel et al")]
-
 // The assembly version has following format:
 //
 // Major.Minor.Build.Revision
 //
 // You can specify all values by your own or you can build default build and revision
 // numbers with the '*' character (the default):
-[assembly: AssemblyVersion("0.4.2.0")]
+[assembly: AssemblyVersion("0.6.0.0")]
+[assembly: Do.ProgramVersion("0.6.1.0", "Release")]
 
 // The following attributes specify the key for the sign of your assembly. See the
 // .NET Framework documentation for more information about signing.
