@@ -25,8 +25,8 @@ namespace Do
 {
 	public class EmptyItem : IItem
 	{
-		public string Name { get { return ""; } }
-		public string Description { get { return ""; } }
-		public string Icon { get { return ""; } }
+		public string Name { get { return string.Empty; } }
+		public string Description { get { return string.Empty; } }
+		public string Icon { get { return string.Empty; } }
 	}
 }
