@@ -23,124 +23,50 @@ using System.Reflection;
 using System.Collections.Generic;
 
 using Do;
+using Do.Addins;
 using Do.Universe;
 
 namespace Do {
 	
+	public class PreferenceChangedEventArgs : EventArgs {
+		public readonly string Key;
+		public readonly object Value;
+		public PreferenceChangedEventArgs (string key, object value)
+		{
+			Key = key; Value = value;
+		}
+	}
+	
 	/// <summary>
 	/// A class providing generic functionality for dealing with preferences,
 	/// whether those preferences are provided by gconf, command line options,
 	/// etc.
 	/// </summary>
-	public class Preferences {
-		const string GConfRootPath = "/apps/gnome-do/preferences/";
-
-		GConf.Client client;
+	public class Preferences : IPreferences {
+	
+		public delegate void PreferenceChangedDelegate (object sender, PreferenceChangedEventArgs args);
+		public event PreferenceChangedDelegate PreferenceChanged;
 		
-		/// <summary>
-		/// Command line options as passed to the application's Main method.
-		/// </summary>
-		/// <param name="args">
-		/// A <see cref="System.String"/> array.
-		/// </param>
-		string [] options;
-
-		/// <summary>
-		/// 
-		/// </summary>
-		/// <param name="args">
-		/// A <see cref="System.String"/>
-		/// </param>
-		public Preferences (string [] args)
+		IPreferencesBackend backend;
+		
+		public Preferences (string name)
+			: this (new GconfPreferencesBackend (name))
 		{
-			client = new GConf.Client();
-			options = args;
 		}
-
-		/// <value>
-		/// The keybinding used to summon Do. Stored as the gconf key
-		/// "key_binding".
-		/// </value>
-		public string SummonKeyBinding
+		
+		public Preferences (IPreferencesBackend backend)
 		{
+			this.backend = backend;
+		}
+		
+		public string this [string key] {
 			get {
-				return Get<string> ("key_binding", "<Super>space");
+				return Get<string> (key, string.Empty);
 			}
 			set {
-				Set<string> ("key_binding", value);
+				Set<string> (key, value);
 			}
 		}
-
-		/// <value>
-		/// Whether Do should use a miniature user interface better suited
-		/// for smaller displays. Enabled with the command line option --mini.
-		/// </value>
-		public bool UseMiniMode
-		{
-			get {
-				return HasOption ("--mini");
-			}
-		}
-		
-		/// <value>
-		/// Whether Do should use a darkframe user interface better suited
-		/// for smaller displays. Enabled with the command line option --mini.
-		/// </value>
-		public bool UseGlassFrame
-		{
-			get {
-				return HasOption ("--glassframe");
-			}
-		}
-
-		/// <value>
-		/// Whether Do should display its window when it first starts. Enable
-		/// with the command line options -q or --quiet.
-		/// </value>
-		public bool BeQuiet {
-			get {
-				return HasOption ("--quiet") || HasOption ("-q");
-			}
-		}
-
-		/// <summary>
-		/// Determines whether an option was among those passed to Do at the
-		/// command line. If Do is started from a command line with these
-		/// options:
-		/// 
-		///   $ gnome-do --quiet --mini
-		/// 
-		/// Only HasOption ("--quiet") and HasOption ("--mini") will be true.
-		/// </summary>
-		/// <param name="option">
-		/// A <see cref="System.String"/> option (e.g. "--quiet).
-		/// </param>
-		/// <returns>
-		/// A <see cref="System.Boolean"/> telling whether the option was
-		/// passed.
-		/// </returns>
-		protected bool HasOption (string option)
-		{
-			return Array.IndexOf (options, option) != -1;
-		}
-
-		/// <summary>
-		/// If key contains an absolute path, return it; otherwise, return
-		/// an absolute path for the key by appending it to Do's root gconf path.
-		/// </summary>
-		/// <param name="key">
-		/// A <see cref="System.String"/> gconf key, containing either an
-		/// absolute path or a key relative to Do's root path (e.g "key_binding"
-		/// or "ui/color").
-		/// </param>
-		/// <returns>
-		/// A <see cref="System.String"/> containing an absolute gconf path.
-		/// </returns>
-		private string MakeKeyPath (string key)
-		{
-			if (key.StartsWith ("/")) return key;
-			return GConfRootPath + key;
-		}
 		
 		/// <summary>
 		/// Sets a gconf key to a given value.
@@ -160,25 +86,20 @@ namespace Do {
 		/// </returns>
 		public bool Set<T> (string key, T val)
 		{
-			bool success;
-
-			success = true;
-			try {
-				client.Set (MakeKeyPath (key), val);
-			} catch {
-				success = false;
+			bool success = backend.Set<T> (key, val);
+			if (success && null != PreferenceChanged) {
+				PreferenceChanged (this,
+					new PreferenceChangedEventArgs (key, val));
 			}
 			return success;
 		}
 
 		/// <summary>
-		/// Read a value stored in Gconf for a given key. Return the default if
+		/// Read a value stored for a given key. Return the default if
 		/// no value exists for that key.
 		/// </summary>
 		/// <param name="key">
-		/// A <see cref="System.String"/> gconf key (e.g. "key_binding") stored
-		/// under Do's root gconf path. You may also specify an absoulte gconf
-		/// path if you want to read any other key.
+		/// A <see cref="System.String"/> key (e.g. "key_binding").
 		/// </param>
 		/// <param name="def">
 		/// A defaukt <see cref="T"/> value to be returned if the key is not
@@ -196,12 +117,10 @@ namespace Do {
 		}
 
 		/// <summary>
-		/// Try to read a value from gconf for a given key.
+		/// Try to read a value for a given key.
 		/// </summary>
 		/// <param name="key">
-		/// A <see cref="System.String"/> gconf key (e.g. "key_binding") stored
-		/// under Do's root gconf path. You may also specify an absoulte gconf
-		/// path if you want to read any other key.
+		/// A <see cref="System.String"/> key (e.g. "key_binding").
 		/// </param>
 		/// <param name="val">
 		/// A <see cref="T"/> value if the key was found.
@@ -212,24 +131,14 @@ namespace Do {
 		/// </returns>
 		public bool TryGet<T> (string key, out T val)
 		{
-			bool success;
-
-			success = true;
-			try {
-				val = (T) client.Get (MakeKeyPath (key));
-			} catch {
-				success = false;
-			}
-			return success;
+			return backend.TryGet<T> (key, out val);
 		}
 		
 		/// <summary>
-		/// Try to read a value from gconf for a given key.
+		/// Try to read a value for a given key.
 		/// </summary>
 		/// <param name="key">
-		/// A <see cref="System.String"/> gconf key (e.g. "key_binding") stored
-		/// under Do's root gconf path. You may also specify an absoulte gconf
-		/// path if you want to read any other key.
+		/// A <see cref="System.String"/> key (e.g. "key_binding").
 		/// </param>
 		/// <param name="def">
 		/// A default <see cref="T"/> value to be returned if the key is not
@@ -254,4 +163,4 @@ namespace Do {
 			return success;
 		}
 	}
-}
+}
\ No newline at end of file
