@@ -28,6 +28,7 @@ using Gdk;
 
 using Mono.Unix;
 using Do.UI;
+using Do.Addins;
 
 namespace Do
 {
@@ -41,6 +42,7 @@ namespace Do
 		{
 			// Misc
 			Addins.Util.FormatCommonSubstrings = FormatCommonSubstrings;
+			Addins.Util.GetPreferences = GetPreferences;
 
 			// Environment utilities
 			Addins.Util.Environment.Open = Environment.Open;
@@ -48,28 +50,32 @@ namespace Do
 			// Appearance utilities			
 			Addins.Util.Appearance.MarkupSafeString = Appearance.MarkupSafeString;
 			Addins.Util.Appearance.PresentWindow = Appearance.PresentWindow;
-			//
 			Addins.Util.Appearance.PopupMainMenuAtPosition = MainMenu.Instance.PopupAtPosition;
 		}
+		
+		public static IPreferences GetPreferences (string id)
+		{
+			return new Preferences (id);
+		}
 
 		public class Environment
 		{
 			public static void Open (string open_item)
 			{
-				Process start_proc;
-
 				if (open_item == null) return;
 
-				start_proc = new Process ();
-				// start_proc.StartInfo.FileName = open_item;
-				// start_proc.StartInfo.UseShellExecute = true;
-				start_proc.StartInfo.FileName = "xdg-open";
-				start_proc.StartInfo.Arguments = open_item;
-				try {
-					Log.Info ("Opening \"{0}\"...", open_item);
-					start_proc.Start ();
-				} catch (Exception e) {
-					Log.Error ("Failed to open {0}: {1}", open_item, e.Message);
+				using (Process start_proc = new Process ())
+				{
+					// start_proc.StartInfo.FileName = open_item;
+					// start_proc.StartInfo.UseShellExecute = true;
+					start_proc.StartInfo.FileName = "xdg-open";
+					start_proc.StartInfo.Arguments = open_item;
+					try {
+						Log.Debug ("Opening \"{0}\"...", open_item);
+						start_proc.Start ();
+					} catch (Exception e) {
+						Log.Error ("Failed to open {0}: {1}", open_item, e.Message);
+					}
 				}
 			}
 		}
@@ -79,9 +85,8 @@ namespace Do
 
 			public static string MarkupSafeString (string s)
 			{
-				if (s == null) return "";
-				s = s.Replace ("&", "&amp;");
-				return s;
+				if (s == null) return string.Empty;
+				return GLib.Markup.EscapeText (s);
 			}
 
 			
@@ -131,7 +136,7 @@ namespace Do
 			string skipped, matched, remainder;
 			bool matchedTermination;
 
-			result = "";
+			result = string.Empty;
 			match_pos = last_main_cut = 0;
 			lower_main = main.ToLower ();
 			other = other.ToLower ();
@@ -160,13 +165,13 @@ namespace Do
 						remainder = FormatCommonSubstrings ( main.Substring (match_pos + len), other.Substring (pos + len), format);
 					}
 					else {
-						remainder = "";
+						remainder = string.Empty;
 					}
 					result = string.Format ("{0}{1}{2}", skipped, string.Format(format, matched), remainder);
 					break;
 				}
 			}
-			if (result == "") {
+			if (result == string.Empty) {
 				// no matches
 				result = main;
 			}
